// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of passive_data;

/// Test if the given type String is a passive data structure.
bool _isPdsType(String type) {
  if (['dynamic', 'num', 'int', 'double', 'bool', 'String'].contains(type)) {
    return true;
  } else {
    if (type.startsWith('List<')) {
      return _isPdsType(type.substring(5, type.length - 1));
    } else if (type.startsWith('Set<')) {
      return _isPdsType(type.substring(4, type.length - 1));
    } else if (type.startsWith('Map<int,')) {
      return _isPdsType(type.substring(8, type.length - 1));
    } else if (type.startsWith('Map<String,')) {
      return _isPdsType(type.substring(11, type.length - 1));
    } else {
      return false;
    }
  }
}

/// Create reverse construction code for the given [type] where [data] is used
/// to access the object that contains the data to construct this type and
/// [jsonType] represents the JSON representation of [type]. [depth] represents
/// the recursive level. This is required to name the List generator iteration
/// index variable.
String generateCodeFromPsd(String type, String data,
    [String pdsType = '', int depth = 1]) {
  // Strip off all spaces.
  type = type.replaceAll(' ', '');

  if (type == 'dynamic') {
    return data;
  } else if (_isPdsType(type)) {
    // Data can be casted directly.
    return '$data as $type';
  } else if ((type.startsWith('List<') || type.startsWith('Set<')) &&
      pdsType.isNotEmpty) {
    // Unwrap the List<...> or Set<...> type.
    var offset = type.startsWith('List<') ? 5 : 4;
    var innerType = type.substring(offset, type.length - 1);
    var innerPdsType = pdsType.substring(offset, pdsType.length - 1);

    // Generate constructor for List items.
    var innerConstructor = generateCodeFromPsd(
        innerType, '$data[i$depth]', innerPdsType, depth + 1);

    return "new List<$innerType>.generate($data.length, (int i$depth) => $innerConstructor)" +
        (type.startsWith('Set<') ? '.toSet()' : '');
  } else {
    // Fallback to .fromPds constructor.
    return pdsType == 'dynamic'
        ? 'new $type.fromPds($data)'
        : 'new $type.fromPds($data as $pdsType)';
  }
}
